@@ -47,6 +47,8 @@ gboolean tracker_data_manager_init                (TrackerDBManagerFlags  flags,
                                                    gpointer               busy_user_data,
                                                    const gchar           *busy_status);
 void     tracker_data_manager_shutdown            (void);
+gboolean tracker_data_manager_interrupt_thread    (GThread               *thread);
+
 gint64   tracker_data_manager_get_db_option_int64 (const gchar           *option);
 void     tracker_data_manager_set_db_option_int64 (const gchar           *option,
                                                    gint64                 value);
@@ -56,18 +58,23 @@ void     tracker_data_ontology_load_statement     (const gchar           *ontolo
                                                    const gchar           *predicate,
                                                    const gchar           *object,
                                                    gint                  *max_id,
-                                                   gboolean               is_new,
+                                                   gboolean               in_update,
                                                    GHashTable            *classes,
-                                                   GHashTable            *properties);
+                                                   GHashTable            *properties,
+                                                   GPtrArray             *seen_classes,
+                                                   GPtrArray             *seen_properties);
 void     tracker_data_ontology_import_into_db     (gboolean               is_new);
 void     tracker_data_ontology_process_statement  (const gchar           *graph,
                                                    const gchar           *subject,
                                                    const gchar           *predicate,
                                                    const gchar           *object,
                                                    gboolean               is_uri,
-                                                   gboolean               is_new,
+                                                   gboolean               in_update,
                                                    gboolean               ignore_nao_last_modified);
 void    tracker_data_ontology_import_finished     (void);
+void    tracker_data_ontology_process_changes     (GPtrArray             *seen_classes,
+                                                   GPtrArray             *seen_properties);
+void    tracker_data_ontology_free_seen           (GPtrArray             *seen);
 
 G_END_DECLS
 
