@@ -346,7 +346,7 @@ tracker_data_update_get_new_service_id (void)
 	g_object_unref (stmt);
 
 	if (cursor) {
-		tracker_db_cursor_iter_next (cursor);
+		tracker_db_cursor_iter_next (cursor, NULL);
 		max_service_id = MAX (tracker_db_cursor_get_int (cursor, 0), max_service_id);
 		g_object_unref (cursor);
 	}
@@ -380,7 +380,7 @@ tracker_data_update_get_next_modseq (void)
 	g_object_unref (stmt);
 
 	if (cursor) {
-		tracker_db_cursor_iter_next (cursor);
+		tracker_db_cursor_iter_next (cursor, NULL);
 		max_modseq = MAX (tracker_db_cursor_get_int (cursor, 0), max_modseq);
 		g_object_unref (cursor);
 	}
@@ -2388,11 +2388,11 @@ ontology_transaction_end (GList *ontology_queue,
 	GList *l;
 	const gchar *ontology_uri = NULL;
 
-	tracker_data_ontology_process_changes (seen_classes, seen_properties);
-
 	/* Perform ALTER-TABLE and CREATE-TABLE calls for all that are is_new */
 	tracker_data_ontology_import_into_db (TRUE);
 
+	tracker_data_ontology_process_changes (seen_classes, seen_properties);
+
 	for (l = ontology_queue; l; l = l->next) {
 		QueuedStatement *queued = ontology_queue->data;
 
