@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 
 #include <sqlite3.h>
 #include <stdlib.h>
+#include <math.h>
 
 #include <libtracker-common/tracker-common.h>
 #include <libtracker-fts/tracker-fts.h>
@@ -40,21 +41,21 @@
 #define TRACKER_DB_STATEMENT_SQLITE(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqlite))
 #define TRACKER_DB_STATEMENT_SQLITE_CLASS(c)     (G_TYPE_CHECK_CLASS_CAST ((c),    TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqliteClass))
 #define TRACKER_IS_DB_STATEMENT_SQLITE(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_TYPE_DB_STATEMENT_SQLITE))
+
 #define TRACKER_IS_DB_STATEMENT_SQLITE_CLASS(c)  (G_TYPE_CHECK_CLASS_TYPE ((o),    TRACKER_TYPE_DB_STATEMENT_SQLITE))
 #define TRACKER_DB_STATEMENT_SQLITE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o),  TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqliteClass))
 
+#define TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE(o) (((TrackerDBInterfaceSqlite *)o)->priv)
+#define TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE_O(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_INTERFACE_SQLITE, TrackerDBInterfaceSqlitePrivate))
 
-#define TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_INTERFACE_SQLITE, TrackerDBInterfaceSqlitePrivate))
-
-#define TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqlitePrivate))
+#define TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE(o) (((TrackerDBStatementSqlite *)o)->priv)
+#define TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE_O(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_STATEMENT_SQLITE, TrackerDBStatementSqlitePrivate))
 
 #define TRACKER_DB_CURSOR_SQLITE_GET_PRIVATE_O(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_DB_CURSOR_SQLITE, TrackerDBCursorSqlitePrivate))
 #define TRACKER_DB_CURSOR_SQLITE_GET_PRIVATE(o) (((TrackerDBCursorSqlite *)o)->priv)
 
-typedef struct TrackerDBInterfaceSqlitePrivate TrackerDBInterfaceSqlitePrivate;
 typedef struct TrackerDBStatementSqlitePrivate TrackerDBStatementSqlitePrivate;
 typedef struct TrackerDBCursorSqlitePrivate TrackerDBCursorSqlitePrivate;
-typedef struct SqliteFunctionData SqliteFunctionData;
 typedef struct TrackerDBCursorSqlite TrackerDBCursorSqlite;
 typedef struct TrackerDBCursorSqliteClass TrackerDBCursorSqliteClass;
 typedef struct TrackerDBStatementSqlite      TrackerDBStatementSqlite;
@@ -75,12 +76,12 @@ struct TrackerDBInterfaceSqlitePrivate {
 	sqlite3 *db;
 
 	GHashTable *dynamic_statements;
-	GHashTable *statements;
 
 	GSList *function_data;
 
 	guint in_transaction : 1;
 	guint ro : 1;
+	guint fts_initialized : 1;
 };
 
 struct TrackerDBStatementSqlitePrivate {
@@ -95,14 +96,9 @@ struct TrackerDBCursorSqlitePrivate {
 	gboolean finished;
 };
 
-struct SqliteFunctionData {
-	TrackerDBInterface *interface;
-	TrackerDBFunc func;
-};
-
-
 struct TrackerDBStatementSqlite {
 	GObject parent_instance;
+	TrackerDBStatementSqlitePrivate *priv;
 };
 
 struct TrackerDBStatementSqliteClass {
@@ -144,9 +140,355 @@ G_DEFINE_TYPE_WITH_CODE (TrackerDBCursorSqlite, tracker_db_cursor_sqlite, G_TYPE
 void
 tracker_db_interface_sqlite_enable_shared_cache (void)
 {
+	sqlite3_config (SQLITE_CONFIG_MULTITHREAD);
 	sqlite3_enable_shared_cache (1);
 }
 
+static void
+function_sparql_string_join (sqlite3_context *context,
+                             int              argc,
+                             sqlite3_value   *argv[])
+{
+	GString *str = NULL;
+	const gchar *separator;
+	gint i;
+
+	/* fn:string-join (str1, str2, ..., separator) */
+
+	if (sqlite3_value_type (argv[argc-1]) != SQLITE_TEXT) {
+		sqlite3_result_error (context, "Invalid separator", -1);
+		return;
+	}
+
+	separator = sqlite3_value_text (argv[argc-1]);
+
+	for (i = 0;i < argc-1; i++) {
+		if (sqlite3_value_type (argv[argc-1]) == SQLITE_TEXT) {
+			const gchar *text = sqlite3_value_text (argv[i]);
+
+			if (text != NULL) {
+				if (!str) {
+					str = g_string_new (text);
+				} else {
+					g_string_append_printf (str, "%s%s", separator, text);
+				}
+			}
+		}
+	}
+
+	if (str) {
+		sqlite3_result_text (context, str->str, str->len, g_free);
+		g_string_free (str, FALSE);
+	} else {
+		sqlite3_result_null (context);
+	}
+
+	return;
+}
+
+/* Create a title-type string from the filename for replacing missing ones */
+static void
+function_sparql_string_from_filename (sqlite3_context *context,
+                                      int              argc,
+                                      sqlite3_value   *argv[])
+{
+	gchar  *name = NULL;
+	gchar  *suffix = NULL;
+
+	if (argc != 1) {
+		sqlite3_result_error (context, "Invalid argument count", -1);
+		return;
+	}
+
+	/* "/home/user/path/title_of_the_movie.movie" -> "title of the movie"
+	 * Only for local files currently, do we need to change? */
+
+	name = g_filename_display_basename (sqlite3_value_text (argv[0]));
+
+	if (!name) {
+		sqlite3_result_null (context);
+		return;
+	}
+
+	suffix = g_strrstr (name, ".");
+
+	if (suffix) {
+		*suffix = '\0';
+	}
+
+	g_strdelimit (name, "._", ' ');
+
+	sqlite3_result_text (context, name, -1, g_free);
+}
+
+static void
+function_sparql_uri_is_parent (sqlite3_context *context,
+                               int              argc,
+                               sqlite3_value   *argv[])
+{
+	const gchar *uri, *parent, *remaining;
+	gboolean match = FALSE;
+	guint parent_len;
+
+	if (argc != 2) {
+		sqlite3_result_error (context, "Invalid argument count", -1);
+		return;
+	}
+
+	parent = sqlite3_value_text (argv[0]);
+	uri = sqlite3_value_text (argv[1]);
+
+	if (!parent || !uri) {
+		sqlite3_result_error (context, "Invalid arguments", -1);
+		return;
+	}
+
+	parent_len = sqlite3_value_bytes (argv[0]);
+
+	/* Check only one argument, it's going to
+	 * be compared with the other anyway.
+	 */
+
+	if (!(parent_len >= 7 && (parent[4] == ':' && parent[5] == '/' && parent[6] == '/'))) {
+		if (strstr (parent, "://") == NULL) {
+			sqlite3_result_int (context, FALSE);
+			return;
+		}
+	}
+
+	/* Remove trailing '/', will
+	 * be checked later on uri.
+	 */
+	while (parent[parent_len - 1] == '/') {
+		parent_len--;
+	}
+
+	if (strncmp (uri, parent, parent_len) == 0 && uri[parent_len] == '/') {
+		const gchar *slash;
+
+		while (uri[parent_len] == '/') {
+			parent_len++;
+		}
+
+		remaining = &uri[parent_len];
+
+		if (*remaining == '\0') {
+			/* Exact match, not a child */
+			match = FALSE;
+		} else if ((slash = strchr (remaining, '/')) == NULL) {
+			/* Remaining doesn't have uri
+			 * separator, it's a direct child.
+			 */
+			match = TRUE;
+		} else {
+			/* Check it's not trailing slashes */
+			while (*slash == '/') {
+				slash++;
+			}
+
+			match = (*slash == '\0');
+		}
+	}
+
+	sqlite3_result_int (context, match);
+}
+
+static void
+function_sparql_uri_is_descendant (sqlite3_context *context,
+                                   int              argc,
+                                   sqlite3_value   *argv[])
+{
+	const gchar *uri, *parent, *remaining;
+	gboolean match = FALSE;
+	guint parent_len;
+
+	if (argc != 2) {
+		sqlite3_result_error (context, "Invalid argument count", -1);
+		return;
+	}
+
+	parent = sqlite3_value_text (argv[0]);
+	uri = sqlite3_value_text (argv[1]);
+
+	if (!parent || !uri) {
+		sqlite3_result_error (context, "Invalid arguments", -1);
+		return;
+	}
+
+	parent_len = sqlite3_value_bytes (argv[0]);
+
+	/* Check only one argument, it's going to
+	 * be compared with the other anyway.
+	 */
+
+	if (!(parent_len >= 7 && (parent[4] == ':' && parent[5] == '/' && parent[6] == '/'))) {
+		if (strstr (parent, "://") == NULL) {
+			sqlite3_result_int (context, FALSE);
+			return;
+		}
+	}
+
+	/* Remove trailing '/', will
+	 * be checked later on uri.
+	 */
+	while (parent[parent_len - 1] == '/') {
+		parent_len--;
+	}
+
+	if (strncmp (uri, parent, parent_len) == 0 && uri[parent_len] == '/') {
+		while (uri[parent_len] == '/') {
+			parent_len++;
+		}
+
+		remaining = &uri[parent_len];
+
+		if (remaining && *remaining) {
+			match = TRUE;
+		}
+	}
+
+	sqlite3_result_int (context, match);
+}
+
+static void
+function_sparql_cartesian_distance (sqlite3_context *context,
+                                    int              argc,
+                                    sqlite3_value   *argv[])
+{
+	gdouble lat1;
+	gdouble lat2;
+	gdouble lon1;
+	gdouble lon2;
+
+	gdouble R;
+	gdouble a;
+	gdouble b;
+	gdouble c;
+	gdouble d;
+
+	if (argc != 4) {
+		sqlite3_result_error (context, "Invalid argument count", -1);
+		return;
+	}
+
+	lat1 = sqlite3_value_double (argv[0])*M_PI/180;
+	lat2 = sqlite3_value_double (argv[1])*M_PI/180;
+	lon1 = sqlite3_value_double (argv[2])*M_PI/180;
+	lon2 = sqlite3_value_double (argv[3])*M_PI/180;
+
+	R = 6371000;
+	a = M_PI/2 - lat1;
+	b = M_PI/2 - lat2;
+	c = sqrt(a*a + b*b - 2*a*b*cos(lon2 - lon1));
+	d = R*c;
+
+	sqlite3_result_double (context, d);
+}
+
+static void
+function_sparql_haversine_distance (sqlite3_context *context,
+                                    int              argc,
+                                    sqlite3_value   *argv[])
+{
+	gdouble lat1;
+	gdouble lat2;
+	gdouble lon1;
+	gdouble lon2;
+
+	gdouble R;
+	gdouble dLat;
+	gdouble dLon;
+	gdouble a;
+	gdouble c;
+	gdouble d;
+
+	if (argc != 4) {
+		sqlite3_result_error (context, "Invalid argument count", -1);
+		return;
+	}
+
+	lat1 = sqlite3_value_double (argv[0])*M_PI/180;
+	lat2 = sqlite3_value_double (argv[1])*M_PI/180;
+	lon1 = sqlite3_value_double (argv[2])*M_PI/180;
+	lon2 = sqlite3_value_double (argv[3])*M_PI/180;
+
+	R = 6371000;
+	dLat = (lat2-lat1);
+	dLon = (lon2-lon1);
+	a = sin(dLat/2) * sin(dLat/2) + cos(lat1) * cos(lat2) *  sin(dLon/2) * sin(dLon/2);
+	c = 2 * atan2(sqrt(a), sqrt(1-a));
+	d = R * c;
+
+	sqlite3_result_double (context, d);
+}
+
+static void
+function_sparql_regex (sqlite3_context *context,
+                       int              argc,
+                       sqlite3_value   *argv[])
+{
+	gboolean ret;
+	const gchar *text, *pattern, *flags;
+	GRegexCompileFlags regex_flags;
+	GRegex *regex;
+
+	if (argc != 3) {
+		sqlite3_result_error (context, "Invalid argument count", -1);
+		return;
+	}
+
+	regex = sqlite3_get_auxdata (context, 1);
+
+	text = sqlite3_value_text (argv[0]);
+	flags = sqlite3_value_text (argv[2]);
+
+	if (regex == NULL) {
+		gchar *err_str;
+		GError *error = NULL;
+
+		pattern = sqlite3_value_text (argv[1]);
+
+		regex_flags = 0;
+		while (*flags) {
+			switch (*flags) {
+			case 's':
+				regex_flags |= G_REGEX_DOTALL;
+				break;
+			case 'm':
+				regex_flags |= G_REGEX_MULTILINE;
+				break;
+			case 'i':
+				regex_flags |= G_REGEX_CASELESS;
+				break;
+			case 'x':
+				regex_flags |= G_REGEX_EXTENDED;
+				break;
+			default:
+				err_str = g_strdup_printf ("Invalid SPARQL regex flag '%c'", *flags);
+				sqlite3_result_error (context, err_str, -1);
+				g_free (err_str);
+				return;
+			}
+			flags++;
+		}
+
+		regex = g_regex_new (pattern, regex_flags, 0, &error);
+
+		if (error) {
+			sqlite3_result_error (context, error->message, -1);
+			g_clear_error (&error);
+			return;
+		}
+
+		sqlite3_set_auxdata (context, 1, regex, (void (*) (void*)) g_regex_unref);
+	}
+
+	ret = g_regex_match (regex, text, 0, NULL);
+
+	sqlite3_result_int (context, ret);
+}
+
+
 static void
 open_database (TrackerDBInterfaceSqlitePrivate *priv)
 {
@@ -166,6 +508,34 @@ open_database (TrackerDBInterfaceSqlitePrivate *priv)
 		}
 	}
 
+	sqlite3_create_function (priv->db, "SparqlRegex", 3, SQLITE_ANY,
+	                         priv, &function_sparql_regex,
+	                         NULL, NULL);
+
+	sqlite3_create_function (priv->db, "SparqlHaversineDistance", 4, SQLITE_ANY,
+	                         priv, &function_sparql_haversine_distance,
+	                         NULL, NULL);
+
+	sqlite3_create_function (priv->db, "SparqlCartesianDistance", 4, SQLITE_ANY,
+	                         priv, &function_sparql_cartesian_distance,
+	                         NULL, NULL);
+
+	sqlite3_create_function (priv->db, "SparqlStringFromFilename", 1, SQLITE_ANY,
+	                         priv, &function_sparql_string_from_filename,
+	                         NULL, NULL);
+
+	sqlite3_create_function (priv->db, "SparqlStringJoin", -1, SQLITE_ANY,
+	                         priv, &function_sparql_string_join,
+	                         NULL, NULL);
+
+	sqlite3_create_function (priv->db, "SparqlUriIsParent", 2, SQLITE_ANY,
+	                         priv, &function_sparql_uri_is_parent,
+	                         NULL, NULL);
+
+	sqlite3_create_function (priv->db, "SparqlUriIsDescendant", 2, SQLITE_ANY,
+	                         priv, &function_sparql_uri_is_descendant,
+	                         NULL, NULL);
+
 	sqlite3_extended_result_codes (priv->db, 0);
 	sqlite3_busy_timeout (priv->db, 100000);
 }
@@ -239,19 +609,24 @@ tracker_db_interface_sqlite_get_property (GObject    *object,
 }
 
 static void
-close_database (TrackerDBInterfaceSqlitePrivate *priv)
+close_database (GObject                         *object,
+                TrackerDBInterfaceSqlitePrivate *priv)
 {
+	gint rc;
+
 	g_hash_table_unref (priv->dynamic_statements);
 	priv->dynamic_statements = NULL;
 
-	g_hash_table_unref (priv->statements);
-	priv->statements = NULL;
-
 	g_slist_foreach (priv->function_data, (GFunc) g_free, NULL);
 	g_slist_free (priv->function_data);
 	priv->function_data = NULL;
 
-	sqlite3_close (priv->db);
+	if (priv->fts_initialized) {
+		tracker_fts_shutdown (object);
+	}
+
+	rc = sqlite3_close (priv->db);
+	g_warn_if_fail (rc == SQLITE_OK);
 }
 
 void
@@ -262,7 +637,8 @@ tracker_db_interface_sqlite_fts_init (TrackerDBInterfaceSqlite *interface,
 
 	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (interface);
 
-	tracker_fts_init (priv->db, create);
+	tracker_fts_init (priv->db, create, G_OBJECT (interface));
+	priv->fts_initialized = TRUE;
 }
 
 static void
@@ -272,7 +648,7 @@ tracker_db_interface_sqlite_finalize (GObject *object)
 
 	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (object);
 
-	close_database (priv);
+	close_database (object, priv);
 
 	g_message ("Closed sqlite3 database:'%s'", priv->filename);
 
@@ -321,10 +697,6 @@ prepare_database (TrackerDBInterfaceSqlitePrivate *priv)
 	priv->dynamic_statements = g_hash_table_new_full (g_str_hash, g_str_equal,
 	                                                  (GDestroyNotify) g_free,
 	                                                  (GDestroyNotify) g_object_unref);
-	priv->statements = g_hash_table_new_full (g_str_hash, g_str_equal,
-	                                          (GDestroyNotify) g_free,
-	                                          (GDestroyNotify) sqlite3_finalize);
-
 }
 
 static void
@@ -332,9 +704,11 @@ tracker_db_interface_sqlite_init (TrackerDBInterfaceSqlite *db_interface)
 {
 	TrackerDBInterfaceSqlitePrivate *priv;
 
-	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (db_interface);
+	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE_O (db_interface);
 
 	priv->ro = FALSE;
+	db_interface->priv = priv;
+
 	prepare_database (priv);
 }
 
@@ -380,75 +754,6 @@ add_row (TrackerDBResultSet *result_set,
 	}
 }
 
-static void
-internal_sqlite3_function (sqlite3_context *context,
-                           int              argc,
-                           sqlite3_value   *argv[])
-{
-	SqliteFunctionData *data;
-	GValue *values, result;
-	gint i;
-
-	data = (SqliteFunctionData *) sqlite3_user_data (context);
-	values = g_new0 (GValue, argc);
-
-	/* Transform the arguments */
-	for (i = 0; i < argc; i++) {
-		switch (sqlite3_value_type (argv[i])) {
-		case SQLITE_TEXT:
-			g_value_init (&values[i], G_TYPE_STRING);
-			g_value_set_string (&values[i], (gchar *) sqlite3_value_text (argv[i]));
-			break;
-		case SQLITE_INTEGER:
-			g_value_init (&values[i], G_TYPE_INT);
-			g_value_set_int (&values[i], sqlite3_value_int (argv[i]));
-			break;
-		case SQLITE_FLOAT:
-			g_value_init (&values[i], G_TYPE_DOUBLE);
-			g_value_set_double (&values[i], sqlite3_value_double (argv[i]));
-			break;
-		case SQLITE_NULL:
-			/* Unset GValues as NULLs */
-			break;
-		default:
-			g_critical ("Unknown sqlite3 database value type:%d",
-			            sqlite3_value_type (argv[i]));
-		}
-	}
-
-	/* Call the function */
-	result = data->func (data->interface, argc, values);
-
-	/* And return something appropriate to the context */
-	if (G_VALUE_HOLDS_INT (&result)) {
-		sqlite3_result_int (context, g_value_get_int (&result));
-	} else if (G_VALUE_HOLDS_DOUBLE (&result)) {
-		sqlite3_result_double (context, g_value_get_double (&result));
-	} else if (G_VALUE_HOLDS_STRING (&result)) {
-		sqlite3_result_text (context,
-		                     g_value_dup_string (&result),
-		                     -1, g_free);
-	} else if (G_VALUE_HOLDS (&result, G_TYPE_INVALID)) {
-		sqlite3_result_null (context);
-	} else {
-		g_critical ("Sqlite3 returned type not managed:'%s'",
-		            G_VALUE_TYPE_NAME (&result));
-		sqlite3_result_null (context);
-	}
-
-	/* Now free all this mess */
-	for (i = 0; i < argc; i++) {
-		if (G_IS_VALUE (&values[i])) {
-			g_value_unset (&values[i]);
-		}
-	}
-
-	if (! G_VALUE_HOLDS (&result, G_TYPE_INVALID)) {
-		g_value_unset (&result);
-	}
-
-	g_free (values);
-}
 
 static TrackerDBStatement *
 tracker_db_interface_sqlite_create_statement (TrackerDBInterface *db_interface,
@@ -480,13 +785,6 @@ tracker_db_interface_sqlite_create_statement (TrackerDBInterface *db_interface,
 	return g_object_ref (stmt);
 }
 
-static void
-foreach_print_error (gpointer key, gpointer value, gpointer stmt)
-{
-	if (value == stmt)
-		g_print ("In %s\n", (char*) key);
-}
-
 static TrackerDBResultSet *
 create_result_set_from_stmt (TrackerDBInterfaceSqlite  *interface,
                              sqlite3_stmt              *stmt,
@@ -522,7 +820,6 @@ create_result_set_from_stmt (TrackerDBInterfaceSqlite  *interface,
 	}
 
 	if (result != SQLITE_DONE) {
-		g_hash_table_foreach (priv->statements, foreach_print_error, stmt);
 
 		/* This is rather fatal */
 		if (sqlite3_errcode (priv->db) == SQLITE_IOERR ||
@@ -606,11 +903,23 @@ tracker_db_interface_sqlite_execute_query (TrackerDBInterface  *db_interface,
 	return result_set;
 }
 
+static gboolean
+tracker_db_interface_sqlite_interrupt (TrackerDBInterface *iface)
+{
+	TrackerDBInterfaceSqlitePrivate *priv;
+
+	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (iface);
+	sqlite3_interrupt (priv->db);
+
+	return TRUE;
+}
+
 static void
 tracker_db_interface_sqlite_iface_init (TrackerDBInterfaceIface *iface)
 {
 	iface->create_statement = tracker_db_interface_sqlite_create_statement;
 	iface->execute_query = tracker_db_interface_sqlite_execute_query;
+	iface->interrupt = tracker_db_interface_sqlite_interrupt;
 }
 
 TrackerDBInterface *
@@ -644,26 +953,6 @@ collation_function (gpointer      data,
 	return (func) ((gchar *) str1, len1, (gchar *) str2, len2);
 }
 
-void
-tracker_db_interface_sqlite_create_function (TrackerDBInterface *interface,
-                                             const gchar        *name,
-                                             TrackerDBFunc       func,
-                                             gint                n_args)
-{
-	SqliteFunctionData *data;
-	TrackerDBInterfaceSqlitePrivate *priv;
-
-	priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (interface);
-
-	data = g_new0 (SqliteFunctionData, 1);
-	data->interface = interface;
-	data->func = func;
-
-	priv->function_data = g_slist_prepend (priv->function_data, data);
-
-	sqlite3_create_function (priv->db, name, n_args, SQLITE_ANY, data, &internal_sqlite3_function, NULL, NULL);
-}
-
 gboolean
 tracker_db_interface_sqlite_set_collation_function (TrackerDBInterfaceSqlite *interface,
                                                     const gchar                      *name,
@@ -878,7 +1167,8 @@ tracker_db_cursor_sqlite_rewind (TrackerDBCursor *cursor)
 }
 
 static gboolean
-tracker_db_cursor_sqlite_iter_next (TrackerDBCursor *cursor)
+tracker_db_cursor_sqlite_iter_next (TrackerDBCursor *cursor,
+                                    GError         **error)
 {
 	TrackerDBCursorSqlitePrivate *priv;
 	priv = TRACKER_DB_CURSOR_SQLITE_GET_PRIVATE (cursor);
@@ -888,6 +1178,23 @@ tracker_db_cursor_sqlite_iter_next (TrackerDBCursor *cursor)
 
 		result = sqlite3_step (priv->stmt);
 
+		if (result == SQLITE_INTERRUPT) {
+			g_set_error (error,
+			             TRACKER_DB_INTERFACE_ERROR,
+			             TRACKER_DB_INTERRUPTED,
+			             "Interrupted");
+		} else if (result != SQLITE_ROW && result != SQLITE_DONE) {
+			TrackerDBStatementSqlite *stmt = priv->ref_stmt;
+			TrackerDBStatementSqlitePrivate *stmt_priv = TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE (stmt);
+			TrackerDBInterfaceSqlite *iface = stmt_priv->db_interface;
+			TrackerDBInterfaceSqlitePrivate *iface_priv = TRACKER_DB_INTERFACE_SQLITE_GET_PRIVATE (iface);
+
+			g_set_error (error,
+			             TRACKER_DB_INTERFACE_ERROR,
+			             TRACKER_DB_QUERY_ERROR,
+			             "%s", sqlite3_errmsg (iface_priv->db));
+		}
+
 		priv->finished = (result != SQLITE_ROW);
 	}
 
@@ -1017,6 +1324,7 @@ tracker_db_cursor_sqlite_iface_init (TrackerDBCursorIface *iface)
 static void
 tracker_db_statement_sqlite_init (TrackerDBStatementSqlite *stmt)
 {
+	stmt->priv = TRACKER_DB_STATEMENT_SQLITE_GET_PRIVATE_O(stmt);
 }
 
 static void
