@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -44,7 +44,10 @@ test_blank (void)
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           NULL,
 	                           NULL,
-	                           FALSE);
+	                           FALSE,
+	                           NULL,
+	                           NULL,
+	                           NULL);
 
 	/* perform update in transaction */
 
